package had;

import javax.swing.JOptionPane;

import had.Kosticka;
import had.Jidlo;

/*
 * Created on 2.11.2005
 * Trida Had
 */

/**
 * @author fsramej1 Trida hada, jeji instance je had, ktereho ovladame
 */

public class Had {
	private Prvek prvni, posledni;

	private boolean aktivni = false;

	public Jidlo ham;

	public static final int DOLEVA = 1;

	public static final int DOPRAVA = 2;

	public static final int NAHORU = 3;

	public static final int DOLU = 4;

	public int smer = DOPRAVA;

	public int psmer = DOPRAVA;

	public boolean konec = true;

	/**
	 * Vrati hodnotu promenne konec, kterou pouzivame ke zjisteni zda had bezi
	 * nebo zda naboural a nebezi.
	 * 
	 * @return Hodnota promenne konec
	 */
	public boolean getKonec() {
		return konec;
	}

	/**
	 * Nastavi hodnotu promenne konec
	 * 
	 * @param konec
	 *            Stav hada (true pokud had naboural a neni aktivni)
	 */
	public void setKonec(boolean konec) {
		this.konec = konec;
	}

	/**
	 * Nastavi hodnotu promenne aktivni, kterou pouzivame ke zjisteni, jestli
	 * had leze nebo jestli byl pozastaven.
	 * 
	 * @param aktivni
	 *            Aktivita hada (false pokud byl pozastaven)
	 */
	public void setAktivni(boolean aktivni) {
		this.aktivni = aktivni;
	}

	/**
	 * Vrati hodnotu promenne aktivni, kterou pouzivame ke zjisteni, jestli had
	 * leze nebo jestli byl pozastaven.
	 * 
	 * @return Stav hada, zda byl pozastaven ci nikoliv
	 */
	public boolean isAktivni() {
		if (aktivni == true) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Vrati hodnotu prommenne smer, cili smer kterym had prave leze
	 * 
	 * @return Hodnota promenne smer (1,2,3 nebo 4)
	 */
	public int getSmer() {
		return smer;
	}

	/**
	 * Nastavi hodnotu prommenne smer, cili smer kterym had prave leze
	 * 
	 * @param smer
	 *            Smer kterym had prave leze
	 */
	public void setSmer(int smer) {
		this.smer = smer;
	}

	/**
	 * Konstruktor hada, pouze nastavuje prazdny spojovy seznam pri vytvoreni
	 * instance
	 *  
	 */
	public Had() {
		prvni = posledni = null;
	}

	/**
	 * Prida do spojoveho seznamu dalsi clanek hada
	 * 
	 * @param o
	 *            Pridavany prvek spojoveho seznamu
	 */
	public void pridej(Object o) {
		if (prvni == null) {
			prvni = new Prvek(o);
			posledni = prvni;
		} else {
			posledni.setDalsi(new Prvek(o));
			posledni = posledni.getDalsi();
		}
	}

	/**
	 * Vezme ze spojoveho seznamu clanek hada s indexem pozice
	 * 
	 * @param pozice
	 *            Pozice zadaneho prvku ve spojovem seznamu
	 * @return Clanek hada ze spojoveho seznamu s indexem pozice
	 */
	public Object get(int pozice) {
		Prvek pom = prvni;
		for (int i = 1; pom != null && i <= pozice; i++) {
			pom = pom.getDalsi();
		}
		if (pom != null) {
			return pom.getHodnota();
		} else {
			return null;
		}
	}

	/**
	 * Smaze ze spojoveho seznamu clanek hada na pozici pozice
	 * 
	 * @param pozice
	 *            Pozice clanku hada ktery chceme smazat
	 */
	public void smaz(int pozice) {
		Prvek pom = prvni;
		if (pozice == 0) {
			prvni = prvni.getDalsi();
			if (prvni == null || prvni.getDalsi() == null) {
				posledni = prvni;
			}
		} else {
			for (int i = 1; pom != null && i < pozice - 1; i++) {
				pom = pom.getDalsi();
			}
			if (pom != null) {
				pom.setDalsi(pom.getDalsi().getDalsi());
				if (pom.getDalsi() == null) {
					posledni = pom;
				}
			}
		}
	}

	/**
	 * Vraci instanci jidla
	 * 
	 * @return Instance jidla
	 */
	public Object getJidlo() {
		return ham;
	}

	/**
	 * Vraci pocet prvku ve spojovem seznamu, cili pocet kosticek hada
	 * 
	 * @return Pocet kosticek hada
	 */
	public int getPocet() {
		Prvek pom = prvni;
		int pocet = 0;

		while (pom != null) {
			pom = pom.getDalsi();
			pocet++;
		}
		return pocet;
	}

	/**
	 * Vraci stav hada vzhledem k sam sobe. Pokud se srazil sam se sebou vrati
	 * true, jinak false
	 * 
	 * @return Pokud se had srazil sam se sebou vraci true, jinak false
	 */
	public boolean srazilSe() {
		Kosticka prvnikosticka, kostka;
		prvnikosticka = (Kosticka) get(0);
		boolean srazil = false;
		if (getPocet() > 4) {
			for (int i = 4; i < getPocet(); i++) {
				kostka = (Kosticka) get(i);
				if ((kostka.getX() == prvnikosticka.getX())
						&& (kostka.getY() == prvnikosticka.getY())) {
					srazil = true;
				}
			}
		}
		return srazil;
	}

	/**
	 * Pohne celym hadem o jednu souradnici doprava
	 *  
	 */
	public void moveDoprava() {
		// Ochrana proti otoceni o 180 stupnu
		if (psmer == DOLEVA) {
			smer = DOLEVA;
			moveDoleva();
		} else {

			int pxlast = 0, pylast = 0;
			smer = DOPRAVA;
			for (int i = 0; i < getPocet(); i++) {
				if (i == 0) {
					Kosticka pom = (Kosticka) get(0);

					int px = pom.getX();
					px++;
					smaz(0);
					if (px < 20) {
						pridej(new Kosticka(px, pom.getY()));
						pxlast = px - 1;
						pylast = pom.getY();
					} else {
						endMessage();
					}
				} else {
					Kosticka pom = (Kosticka) get(0);

					int px = pom.getX();
					px++;
					smaz(0);
					pridej(new Kosticka(pxlast, pylast));
					pxlast = px - 1;
					pylast = pom.getY();
				}

			}
			psmer = DOPRAVA;
		}
		if (srazilSe()) {
			endMessage();
		}
	}

	/**
	 * Pohne celym hadem o jednu souradnici doleva
	 *  
	 */
	public void moveDoleva() {
		// Ochrana proti otoceni o 180 stupnu
		if (psmer == DOPRAVA) {
			smer = DOPRAVA;
			moveDoprava();
		} else {

			int pxlast = 0, pylast = 0;
			smer = DOLEVA;
			for (int i = 0; i < getPocet(); i++) {
				if (i == 0) {
					Kosticka pom = (Kosticka) get(0);

					int px = pom.getX();
					px--;
					smaz(0);
					if (px > 0) {
						pridej(new Kosticka(px, pom.getY()));
						pxlast = px + 1;
						pylast = pom.getY();
					} else {
						endMessage();
					}
				} else {
					Kosticka pom = (Kosticka) get(0);

					int px = pom.getX();
					px--;
					smaz(0);
					pridej(new Kosticka(pxlast, pylast));
					pxlast = px + 1;
					pylast = pom.getY();
				}

			}
			psmer = DOLEVA;
		}
		if (srazilSe()) {
			endMessage();
		}
	}

	/**
	 * Pohne celym hadem o jednu souradnici nahoru
	 *  
	 */
	public void moveNahoru() {
		// Ochrana proti otoceni o 180 stupnu
		if (psmer == DOLU) {
			smer = DOLU;
			moveDolu();
		} else {

			int pxlast = 0, pylast = 0;
			smer = NAHORU;
			for (int i = 0; i < getPocet(); i++) {
				if (i == 0) {
					Kosticka pom = (Kosticka) get(0);

					int py = pom.getY();
					py--;
					smaz(0);
					if (py > 0) {
						pridej(new Kosticka(pom.getX(), py));
						pxlast = pom.getX();
						pylast = py + 1;
					} else {
						endMessage();
					}
				} else {
					Kosticka pom = (Kosticka) get(0);

					int py = pom.getY();
					py--;
					smaz(0);
					pridej(new Kosticka(pxlast, pylast));
					pxlast = pom.getX();
					pylast = py + 1;
				}

			}
			psmer = NAHORU;
		}
		if (srazilSe()) {
			endMessage();
		}
	}

	/**
	 * Pohne celym hadem o jednu souradnici dolu
	 *  
	 */
	public void moveDolu() {
		// Ochrana proti otoceni o 180 stupnu
		if (psmer == NAHORU) {
			smer = NAHORU;
			moveNahoru();
		} else {

			int pxlast = 0, pylast = 0;
			smer = DOLU;
			for (int i = 0; i < getPocet(); i++) {
				if (i == 0) {
					Kosticka pom = (Kosticka) get(0);

					int py = pom.getY();
					py++;
					smaz(0);
					if (py < 15) {
						pridej(new Kosticka(pom.getX(), py));
						pxlast = pom.getX();
						pylast = py - 1;
					} else {
						endMessage();
					}
				} else {
					Kosticka pom = (Kosticka) get(0);

					int py = pom.getY();
					py++;
					smaz(0);
					pridej(new Kosticka(pxlast, pylast));
					pxlast = pom.getX();
					pylast = py - 1;
				}

			}
			psmer = DOLU;
		}
		if (srazilSe()) {
			endMessage();
		}
	}

	/**
	 * Uplne vymaze cely spojovy senam, cili smaze z hada vsechny kosticky
	 *  
	 */
	public void vymaz() {
		prvni = posledni = null;
	}

	/**
	 * Zobrazi chybovou hlasku o nabourani a skonci hru
	 *  
	 */
	private void endMessage() {
		konec = true; // ukonceni pohybu hada
		vymaz(); // vyprazdneni hada pro novou hru
		JOptionPane.showMessageDialog(null, "Konec hry !", "Konec hry !",
				JOptionPane.ERROR_MESSAGE);
	}
}